@@ -18,7 +18,10 @@ from pycamunda.request import QueryParameter, PathParameter, BodyParameter
 URL_SUFFIX = '/process-instance'
 
 
-__all__ = ['Delete', 'GetActivityInstance', 'GetList', 'Get', 'Modify', 'Activate', 'Suspend']
+__all__ = [
+    'Delete', 'GetActivityInstance', 'GetList', 'Get', 'Modify', 'Activate', 'Suspend',
+    'VariablesDelete', 'VariablesGet', 'VariablesGetList', 'VariablesModify', 'VariablesUpdate'
+]
 
 
 @dataclasses.dataclass
@@ -573,3 +576,208 @@ class Suspend(_ActivateSuspend):
             id_=id_,
             suspended=True
         )
+
+
+class VariablesDelete(pycamunda.base.CamundaRequest):
+
+    process_instance_id = PathParameter('id')
+    var_name = PathParameter('varName')
+
+    def __init__(self, url: str, process_instance_id: str, var_name: str):
+        """Delete a process instance variable.
+
+        :param url: Camunda Rest engine URL.
+        :param process_instance_id: Id of the process instance.
+        :param var_name: Name of the variable.
+        """
+        super().__init__(url=url + URL_SUFFIX + '/{id}/variables/{varName}')
+        self.process_instance_id = process_instance_id
+        self.var_name = var_name
+
+    def __call__(self, *args, **kwargs) -> None:
+        """Send the request."""
+        super().__call__(pycamunda.base.RequestMethod.DELETE, *args, **kwargs)
+
+
+class VariablesGet(pycamunda.base.CamundaRequest):
+
+    process_instance_id = PathParameter('id')
+    var_name = PathParameter('varName')
+    deserialize_value = QueryParameter('deserializeValue')
+
+    def __init__(
+        self,
+        url: str,
+        process_instance_id: str,
+        var_name: str,
+        deserialize_value: bool = False,
+        binary: bool = False
+    ):
+        """Get a variable of a process instance.
+
+        :param url: Camunda Rest engine URL.
+        :param process_instance_id: Id of the process instance.
+        :param var_name: Name of the variable.
+        :param deserialize_value: Whether serializable variable values are deserialized on server
+                                  side.
+        :param binary: Whether the requested variable is a binary array or file variable.
+        """
+        super().__init__(url=url + URL_SUFFIX + '/{id}/variables/{varName}')
+        self.process_instance_id = process_instance_id
+        self.var_name = var_name
+        self.deserialize_value = deserialize_value
+        self.binary = binary
+
+    @property
+    def url(self):
+        return super().url + ('/data' if self.binary else '')
+
+    def __call__(self, *args, **kwargs) -> typing.Union[pycamunda.variable.Variable, bytes]:
+        """Send the request."""
+        response = super().__call__(pycamunda.base.RequestMethod.GET, *args, **kwargs)
+
+        if self.binary:
+            return response.content
+        return pycamunda.variable.Variable.load(data=response.json())
+
+
+class VariablesGetList(pycamunda.base.CamundaRequest):
+
+    process_instance_id = PathParameter('id')
+    deserialize_values = QueryParameter('deserializeValues')
+
+    def __init__(
+        self,
+        url: str,
+        process_instance_id: str,
+        deserialize_values: bool = False
+    ):
+        """Get variables of a process instance.
+
+        :param url: Camunda Rest engine URL.
+        :param process_instance_id: Id of the process instance.
+        :param deserialize_values: Whether serializable variable values are deserialized on server
+                                   side.
+        """
+        super().__init__(url=url + URL_SUFFIX + '/{id}/variables')
+        self.process_instance_id = process_instance_id
+        self.deserialize_values = deserialize_values
+
+    def __call__(self, *args, **kwargs) -> typing.Dict[str, pycamunda.variable.Variable]:
+        """Send the request."""
+        response = super().__call__(pycamunda.base.RequestMethod.GET, *args, **kwargs)
+
+        return {
+            name: pycamunda.variable.Variable.load(data=var_json)
+            for name, var_json in response.json().items()
+        }
+
+
+class VariablesModify(pycamunda.base.CamundaRequest):
+
+    process_instance_id = PathParameter('id')
+    modifications = BodyParameter('modifications')
+    deletions = BodyParameter('deletions')
+
+    def __init__(self, url: str, process_instance_id: str, deletions: typing.Iterable[str] = None):
+        """Modify variables of a process instance. This can be either updating or deleting
+        variables.
+
+        :param url: Camunda Rest engine URL.
+        :param process_instance_id: Id of the process instance.
+        :param deletions: Variables to delete.
+        """
+        super().__init__(url=url + URL_SUFFIX + '/{id}/variables')
+        self.process_instance_id = process_instance_id
+        self.deletions = deletions
+
+        self.modifications = {}
+
+    def add_variable(
+            self, name: str, value: typing.Any, type_: str = None, value_info: typing.Any = None
+    ) -> None:
+        """Add a variable to modify.
+
+        :param name: Name of the variable.
+        :param value: Value of the variable.
+        :param type_: Value type of the variable.
+        :param value_info: Additional information regarding the value type.
+        """
+        self.modifications[name] = {'value': value, 'type': type_, 'valueInfo': value_info}
+
+    def body_parameters(self, apply: typing.Callable = ...) -> typing.Dict[str, typing.Any]:
+        params = super().body_parameters(apply=apply)
+        deletions = params.get('deletions', [])
+        if isinstance(deletions, str):
+            params['deletions'] = [deletions]
+        else:
+            params['deletions'] = list(deletions)
+        return params
+
+    def __call__(self, *args, **kwargs) -> None:
+        """Send the request."""
+        super().__call__(pycamunda.base.RequestMethod.POST, *args, **kwargs)
+
+
+class VariablesUpdate(pycamunda.base.CamundaRequest):
+
+    process_instance_id = PathParameter('id')
+    var_name = PathParameter('varName')
+    value = BodyParameter('value')
+    type_ = BodyParameter('type')
+    value_info = BodyParameter('valueInfo')
+
+    def __init__(
+        self,
+        url: str,
+        process_instance_id: str,
+        var_name: str,
+        value: typing.Any, type_: str = None,
+        value_info: typing.Any = None
+    ):
+        """Update a process instance variable. May be used with binary and file variables.
+
+        :param url: Camunda Rest engine URL.
+        :param process_instance_id: Id of the process instance.
+        :param var_name: Name of the variable.
+        :param value: Value of the variable.
+        :param type_: Value type of the variable. To send binary variables use the value 'Bytes' and
+                      to send the binary value of a file variable use the value 'File' for this
+                      parameter.
+        :param value_info: Additional information regarding the value type. Ignored if 'type_' is
+                           'Bytes' or 'File'.
+        """
+        super().__init__(url=url + URL_SUFFIX + '/{id}/variables/{varName}')
+        self.process_instance_id = process_instance_id
+        self.var_name = var_name
+        self.value = value
+        self.type_ = type_
+        self.value_info = value_info
+
+    def _is_binary(self):
+        return self.type_ in ('File', 'Bytes')
+
+    @property
+    def url(self):
+        return super().url + ('/data' if self._is_binary() else '')
+
+    def body_parameters(self, apply: typing.Callable = ...):
+        if self._is_binary():
+            return {'valueType': self.type_}
+        return super().body_parameters(apply=apply)
+
+    @property
+    def files(self):
+        if self._is_binary():
+            return {'data': self.value}
+        return {}
+
+    def __call__(self, *args, **kwargs) -> None:
+        """Send the request."""
+        if self._is_binary():
+            response = super().__call__(pycamunda.base.RequestMethod.POST, *args, **kwargs)
+        else:
+            response = super().__call__(pycamunda.base.RequestMethod.PUT, *args, **kwargs)
+
+        if not response:
+            pycamunda.base._raise_for_status(response)
